%% function stabProp computes the robust stability:
%
%	Inputs:	 -Ci(1x8) : cell array of all eight controllers   % 					    (i:1,...,6,s,m)
%			 -Zi(1x4) : cell array of all four impedances (w.r.t. pos)
%  					    (i:e,h,s,m)
%			 -s 	  : Laplace variable (j\omega). Can either be one %					    frequency or a vector with frequencies
%	Outputs: -stabprop: scalar or vector, depending on input s with 1 %	                    (stable) or 0 (unstable)
%
%%
function stabProp = RobustStability(C,Z,s)

r = 0.075;

kemin = 80*r^2;
kemax = 3000*r^2;
keN = kemin + (kemax-kemin)/2;
ZeN = keN./s;

wzh = 98.7;
zh = 0.49;
wph = 87.31;

Q = -(s./wzh+1)./(s.^2/wph^2+2*zh*s./wph+1);

mh = 0.53*r^2;
bh = 8.58*r^2;
kh = (584.55+(994.20-584.55)/2)*r^2;
ZhN = Q.*(mh.*s + bh + kh./s);

[y11 y12 y21 y22] = extractY(Z,C,s);

% A = (1-y22.*s.*ZeN).^-1;
% B = (1+y11.*s.*ZhN).^-1;
% C = (1+A.*y21.*s.*ZhN.*B.*y12.*s.*ZeN).^-1;
% S1 = C.*A.*(y22 - y21.*s.*ZhN.*B.*y12);
% % % S2 = C.*A.*y21.*(Zh.*y12 - 1).*Q;
% % % T1 = B.*y12.*(1+Ze.*S1);
% % % T2 = B.*y12.*(Ze.*S2-1).*Q;

A = y22 + y21.*(1-s.*ZhN.*y11).^-1.*s.*ZhN.*y12;
S1 = (1-A.*s.*ZeN).^-1.*A;

Wke = kemax-keN;

M11 = Wke.*S1;

stabProp = abs(M11) < 1;

end

