%%
%   evaluateStability
%
%%

function evaluateStability(action)
global data

switch action
    case 'matrices'
        
        s = data.dynamics.S;
        
        Z = data.dynamics.Z;
        C = data.dynamics.C;
        
        data.stabilityproperties.stabProp = evaluateCriterion(C,Z,s);
        
    case 'final'
        
        Freq = data.parameters.Freq;
        
        if(data.discretisation.unitsFreq.Value == 1)
            s = 1j.*2.*pi.*Freq;
        else
            s = 1j.*Freq;
        end
        
        [numZ,denZ,numC,denC] = fixDynamics(0);
        
        n = length(data.dynamics.elements);
        nZ = 4;
        
        for iC = 1:1:n
            if( iC <= nZ )
                %Element is impedance
                Z{iC} = (eval(numZ{iC}))./(eval(denZ{iC}));
            else
                %Element is controller
                C{iC-4} = (eval(numC{iC-4}))./(eval(denC{iC-4}));
            end
        end
        
        stabProp = evaluateCriterion(C,Z,s);
        
        if(~min(stabProp))
            set(data.handles.stabilityindicator,'String','not satisfied',...
                'ForegroundColor',[1 0 0],'Visible','on')
            set(data.handles.stabilitycriteriontext,'Visible','on');
            set(data.handles.stabilitycriteriontext2,'Visible','on');
            lhfound = 0;
            k = 1;
            low = [];
            high = [];
            for i = 1:1:length(stabProp)
                if(stabProp(i) == 0 && lhfound ~= -1)
                    low(k) = i;
                    lhfound = -1;
                end
                if(stabProp(i) == 1 && lhfound == -1)
                    high(k) = i;
                    lhfound = 1;
                    k = k + 1;
                end
            end
            if(length(low) > length(high))
                high(k) = inf;
            end
            
            if(data.discretisation.unitsFreq.Value == 1)
                if(data.plot.unitsFreq.Value == 2)
                    Freq = Freq.*(2.*pi);
                end
            else
                if(data.plot.unitsFreq.Value == 1)
                    Freq = Freq./(2.*pi);
                end
            end
            
            plotString = '';
            ControllerStable = {};
            for i = 1:1:length(low)                
                if(i > 1)
                    plotString = strcat(plotString,'and',{' '});
                end
                if(high(i) == inf)
                    upperbound = 'inf';
                    highSave = length(Freq);
                else
                    upperbound = char(vpa(Freq(high(i)),2));
                    highSave = high(i);
                end
                plotString = strcat(plotString,'[',char(vpa(Freq(low(i)),2)),',',{' '},...
                    upperbound,']',{' '},data.plot.unitsFreq.String{1},{' '});                
                ControllerStable{i} = {low(i),highSave};

            end
            
            set(data.handles.stabilityrangeindicator,'String',plotString,'Visible','on');
            
            data.Design{data.CurController}.ControllerStable = ControllerStable;
            
        else
            set(data.handles.stabilitycriteriontext,'Visible','on');
            set(data.handles.stabilitycriteriontext2,'Visible','off');
            set(data.handles.stabilityindicator,'String','satisfied',...
                'ForegroundColor',[0 1 0],'Visible','on')
            set(data.handles.stabilityrangeindicator,'Visible','off')
            data.Design{data.CurController}.ControllerStable = {};
            
        end
        
end



return


function stabProp = evaluateCriterion(C,Z,s)

global data

switch data.stabilitycriterion
    case 'Default'
        errorHandling('This call cannot be made and some earlier error handling is failing');
    case 'BEP'
        stabProp = BoundedEnvironmentPassivity(C,Z,s);
    case 'LAST'
        stabProp = LlewellynAbsoluteStability(C,Z,s);
    case 'SIES'
        stabProp = NyquistInfiniteEnvStiffness(C,Z,s);
    case 'BES'
        stabProp = BoundedEnvironmentStability(C,Z,s);
    case 'RPT'
        stabProp = RaisbeckPassivityCriterion(C,Z,s);
    case 'OSC'
        f = str2func(data.stabilitycriterionFunction);
        stabProp = f(C,Z,s);
        
end

return

