%%
%
%
%%
function plotMPOmega(action,hObject,eventdata,handles)
global data

switch action
    case 'slider'
        set(data.handles.MagPhaseperSlider,'Enable','off');
end

Mag   = data.parameters.Mag;
Phase = data.parameters.Phase;

if(data.discretisation.unitsFreq.Value == 1)
    % Freq vector is in Hz
    if(data.plot.unitsFreq.Value == 1)
        minS = num2str(10^get(data.handles.MagPhaseperSlider,'Min'));
        maxS = num2str(10^get(data.handles.MagPhaseperSlider,'Max'));
        valueS = num2str(10^get(data.handles.MagPhaseperSlider,'Value'));
        Freq = data.parameters.Freq;
    else
        minS = num2str(10^get(data.handles.MagPhaseperSlider,'Min')*2*pi);
        maxS = num2str(10^get(data.handles.MagPhaseperSlider,'Max')*2*pi);
        valueS = num2str(10^get(data.handles.MagPhaseperSlider,'Value')*2*pi);
        Freq = data.parameters.Freq.*2.*pi;
    end
else
    % Freq vector is in rad/s
    if(data.plot.unitsFreq.Value == 1)
        minS = num2str(10^get(data.handles.MagPhaseperSlider,'Min')/(2*pi));
        maxS = num2str(10^get(data.handles.MagPhaseperSlider,'Max')/(2*pi));
        valueS = num2str(10^get(data.handles.MagPhaseperSlider,'Value')/(2*pi));
        Freq = data.parameters.Freq./(2.*pi);
    else
        minS = num2str(10^get(data.handles.MagPhaseperSlider,'Min'));
        maxS = num2str(10^get(data.handles.MagPhaseperSlider,'Max'));
        valueS = num2str(10^get(data.handles.MagPhaseperSlider,'Value'));
        Freq = data.parameters.Freq;
    end
end

delete(allchild(data.handles.MagPhaseperOmega));

nMag  = length(Mag);
nPhase = length(Phase);

stabProp = data.stabilityproperties.stabProp;
MagPhase = zeros(nMag,nPhase);
MagPerFreq = [];
PhasePerFreq = [];
freqSlider = 10^(get(data.handles.MagPhaseperSlider,'Value'));

switch action
    case 'plot2D'
        set(data.handles.Fhigh,'String',[num2str(maxS),' ',...
            data.plot.unitsFreq.String{1}]);
        set(data.handles.Flow,'String',[num2str(minS),' ',...
            data.plot.unitsFreq.String{1}]);
        set(data.handles.MPfreq,'String',[num2str(valueS),' ',...
            data.plot.unitsFreq.String{1}]);
        data.parameters.plotFreq = 1;
        set(data.handles.MagPhaseperSlider,'Min',log10(Freq(1)));
        set(data.handles.MagPhaseperSlider,'Max',log10(Freq(end)));
        set(data.handles.MagPhaseperSlider,'Value',get(data.handles.MagPhaseperSlider,'Min'));
        set(data.handles.MPfreq,'String',[num2str(10^(get(data.handles.MagPhaseperSlider,'Min'))),' ',...
            data.plot.unitsFreq.String{1}]);
        
    case 'slider'
        [~,i] = min(abs(Freq-freqSlider));
        
        if(i == data.parameters.plotFreq)
            if(freqSlider > Freq(i))
                i = i + 1;
            else
                i = i - 1;
            end
        end
        if(i <= 0)
            i = 1;
        elseif(i > length(Freq))
            i = length(Freq);
        else
        end
        set(data.handles.MagPhaseperSlider,'Value',log10(Freq(i)));
        data.parameters.plotFreq = i;
        set(data.handles.MPfreq,'String',[num2str(Freq(i)),' ',data.plot.unitsFreq.String{1}]);
end

MagPhase(:,:) = stabProp(data.parameters.plotFreq,:,:);

%% Plotting
axes(data.handles.MagPhaseperOmega)
set(data.handles.MagPhaseperOmega,'Box','off')
k = 1;
for iM = 1:1:nMag
    for iP = 1:1:nPhase
        if(MagPhase(iM,iP))
            MagPerFreq(k) = Mag(iM);
            PhasePerFreq(k) = Phase(iP);
            k = k + 1;
        end
    end
end

data.plot.Data.MagPerFreq2D = MagPerFreq;
data.plot.Data.PhasePerFreq2D = PhasePerFreq;

Mmin = data.discretisation.minMag;
Mmax = data.discretisation.maxMag;
Pmin = data.discretisation.minPhase;
Pmax = data.discretisation.maxPhase;

dM = (Mmax - Mmin)/4;
dP = (Pmax - Pmin)/4;
data.plot.scaleMag2D = Mmin:dM:Mmax;
data.plot.scalePhase2D = Pmin:dP:Pmax;

if(data.discretisation.unitsPhase.Value == 1)
    if(data.plot.unitsPhase.Value == 1)
        scalephase = data.plot.scalePhase2D;
        plotphase = PhasePerFreq;
    else
        scalephase = data.plot.scalePhase2D./(360).*(2.*pi);
        plotphase = PhasePerFreq./(360).*(2.*pi);
    end
else
    if(data.plot.unitsPhase.Value == 1)
        scalephase = data.plot.scalePhase2D.*360./(2.*pi);
        plotphase = PhasePerFreq.*360./(2.*pi);
    else
        scalephase = data.plot.scalePhase2D;
        plotphase = PhasePerFreq;
    end
end

plot(plotphase,MagPerFreq,'s','MarkerFaceColor',[0.6 0.6 0.6],...
    'MarkerEdgeColor','none','MarkerSize',1.5,'LineWidth',1);hold on;

set(get(data.handles.MagPhaseperOmega,'YLabel'),'String',...
    ['Magnitude [',data.plot.unitsMag.String{1},']'],'Color',[0 0 0]);
set(get(data.handles.MagPhaseperOmega,'XLabel'),'String',...
    ['Phase [',data.plot.unitsPhase.String{1},']'],'Color',[0 0 0])

set(data.handles.MagPhaseperOmega,'XLim',[scalephase(1)...
    scalephase(end)],'YLim',[data.plot.scaleMag2D(1)...
    data.plot.scaleMag2D(end)],'XTick',scalephase,'YTick',...
    data.plot.scaleMag2D)


%% Plot rectangles
if(data.plot.rectanglesFlag == 1)
    if(data.discretisation.discretisationmethod == 0)
        noplot2 = 0;
        
        if(~isempty(data.stabilityproperties.Mlow{data.parameters.plotFreq}))
            Mlow(1,:) = data.stabilityproperties.Mlow{data.parameters.plotFreq};
        else
            noplot2 = 1;
        end
        if(~isempty(data.stabilityproperties.Mupp{data.parameters.plotFreq}))
            Mupp(1,:) = data.stabilityproperties.Mupp{data.parameters.plotFreq};
        else
            noplot2 = 1;
        end
        
        
        if(~isempty(data.stabilityproperties.Plow{data.parameters.plotFreq}))
            Plow = data.stabilityproperties.Plow{data.parameters.plotFreq};
        else
            noplot2 = 1;
        end
        if(~isempty(data.stabilityproperties.Pupp{data.parameters.plotFreq}))
            Pupp = data.stabilityproperties.Pupp{data.parameters.plotFreq};
        else
            noplot2 = 1;
        end
        
        if(~noplot2)
            if(data.discretisation.unitsPhase.Value == 1)
                if(data.plot.unitsPhase.Value == 2)
                    Plow = Plow./360.*(2.*pi);
                    Pupp = Pupp./360.*(2.*pi);
                end
            else
                if(data.plot.unitsPhase.Value == 1)
                    Plow = Plow./(2.*pi).*360;
                    Pupp = Pupp./(2.*pi).*360;
                end
            end
            
            NTR(1,:) = data.stabilityproperties.NTR{data.parameters.plotFreq};
            rev = data.stabilityproperties.rev{data.parameters.plotFreq};
            
            for imrk = [1 2]
                if rev == 1
                    imrk2 = imrk*-1+3;
                else
                    imrk2 = imrk;
                end
                if rev == 0 || rev == 1
                    
                    Ntr = NTR(imrk2);
                    if Ntr > 0
                        Mlow2 = Mlow(:,imrk2);
                        Mupp2 = Mupp(:,imrk2);
                        Plow2 = Plow(:,imrk2);
                        Pupp2 = Pupp(:,imrk2);
                    end
                    
                    for iNTR = 1:Ntr
                        if(Pupp2(iNTR)-Plow2(iNTR) > 0 && Mupp2-Mlow2 > 0)
                            rectangle('Position',[Plow2(iNTR) Mlow2 Pupp2(iNTR)-Plow2(iNTR) Mupp2-Mlow2],'LineWidth',1,'LineStyle','--')
                        end
                    end
                end
            end
        end
    else
        Prect = Phase;
        if(data.discretisation.unitsPhase.Value == 1)
            if(data.plot.unitsPhase.Value == 2)
                Prect = Phase./360.*(2.*pi);
            end
        else
            if(data.plot.unitsPhase.Value == 1)
                Prect = Phase./(2.*pi).*360;
            end
        end
        
        fourCornersOut = data.stabilityproperties.fourCorners{data.parameters.plotFreq}{1};
        fourCornersIn = data.stabilityproperties.fourCorners{data.parameters.plotFreq}{2};
        
        noRegions = length(fourCornersOut);
        for iNoReg = 1:1:noRegions
            
            P1Out = Prect(fourCornersOut{iNoReg}(1));
            M1Out = Mag(fourCornersOut{iNoReg}(4));
            P2Out = Prect(fourCornersOut{iNoReg}(2));
            M2Out = Mag(fourCornersOut{iNoReg}(3));
            P1In  = Prect(fourCornersIn{iNoReg}(1));
            M1In  = Mag(fourCornersIn{iNoReg}(4));
            P2In  = Prect(fourCornersIn{iNoReg}(2));
            M2In  = Mag(fourCornersIn{iNoReg}(3));
            
            wIn = abs(P2In-P1In);
            wOut = abs(P2Out-P1Out);
            hIn = abs(M2In-M1In);
            hOut = abs(M2Out-M1Out);
            
            if(wIn <= 0)
                wIn = 1;
            end
            if(wOut <= 0)
                wOut = 1;
            end
            if(hIn <= 0)
                hIn = 1;
            end
            if(hOut <= 0)
                hOut = 1;
            end
            
            rectOut = [P1Out,M1Out,wOut,hOut];
            rectIn = [P1In,M1In,wIn,hIn];
            
            rectangle('Position',rectOut,'LineWidth',1,'LineStyle','--');hold on;
            rectangle('Position',rectIn,'LineWidth',1,'LineStyle','--');hold on;
        end
    end
end

%% plot line
if(data.plot.showinbodeFlag)
    plotLineOnFreq();
end

%% Plot controller
switch action
    case {'slider','rectangles'}
        k = 1;
        showWhichController = [];
        for i = 1:1:length(data.plotController)
            if(data.plotController(i) == 1)
                showWhichController(k) = i;
                k = k + 1;
            end
        end
        if(~isempty(showWhichController))
            for i = 1:1:length(showWhichController)
                j = showWhichController(i);
                Controller = data.symController{j};
                
                if(data.discretisation.unitsFreq.Value == 1)
                    s = 1j.*2.*pi.*data.parameters.Freq(data.parameters.plotFreq);
                else
                    s = 1j.*data.parameters.Freq(data.parameters.plotFreq);
                end
                
                if(isnumeric(Controller))
                    ControllerPoint = Controller;
                else
                    ControllerPoint = eval(Controller);
                end
                
                
                MagPoint = db(ControllerPoint);
                
                plotColorMPperF = 'k';
                if(~isempty(data.Design{j}.ControllerStable))
                    
                    for k = 1:1:length(data.Design{j}.ControllerStable)
                        low = data.Design{j}.ControllerStable{k}{1};
                        high = data.Design{j}.ControllerStable{k}{2};
                        if(data.parameters.plotFreq >= low && data.parameters.plotFreq <= high-1)
                            plotColorMPperF = 'r';
                        end
                    end
                end
                
                if(data.plot.unitsPhase.Value == 1)
                    PhasePoint = angle(ControllerPoint)./(2.*pi).*360;
                else
                    PhasePoint = angle(ControllerPoint);
                end
                axes(data.handles.MagPhaseperOmega)
                data.plot.Pointcontroller{j} =...
                    plot(PhasePoint,MagPoint,'.','Color',plotColorMPperF,'LineWidth',400);hold on;
            end
        end
        set(data.handles.MagPhaseperSlider,'Enable','on');
end

if(data.plot.externalPlot == 1)
    g = figure;
    xlabel(strcat('Phase (',data.plot.unitsPhase.String,')'))
    ylabel(strcat('Magnitude (',data.plot.unitsMag.String,')'));
    title(strcat('Frequency = ',get(data.handles.MPfreq,'String')));

    copyobj(get(data.handles.MagPhaseperOmega,'children'),gca);
    set(gca,'XTickLabel',get(data.handles.MagPhaseperOmega,'XTickLabel'),...
        'XTick',get(data.handles.MagPhaseperOmega,'XTick'),...
        'XLim',get(data.handles.MagPhaseperOmega,'XLim'),...
        'XScale',get(data.handles.MagPhaseperOmega,'XScale'),...
        'XColor',get(data.handles.MagPhaseperOmega,'XColor'),...
        'YTickLabel',get(data.handles.MagPhaseperOmega,'YTickLabel'),...
        'YTick',get(data.handles.MagPhaseperOmega,'YTick'),...
        'YLim',get(data.handles.MagPhaseperOmega,'YLim'),...
        'box','on','layer','top');
end


% function [Result] = resetRange(element,eEnd,Vector)
%
% Result = 0;
% if(element == 1)
%     Result = Vector(1)-Vector(1)/10;
% elseif(element == eEnd)
%     Result = Vector(eEnd)+Vector(eEnd)*10;
% else
%     Result = Vector(element);
% end
%
% return